import asyncio
from datetime import datetime
import json
from pprint import pprint

import aiohttp
from aiohttp import ClientSession

import urllib3

from joblib import Parallel, delayed, parallel_backend

from requests import Request, Session, request
from requests_toolbelt import sessions

from qantio.sdk.models.TimeSerie import TimeSerie
from qantio.sdk.models.OperationResult import OperationResult
from qantio.sdk.models.BatchOperationResult import BatchOperationResult
from qantio.sdk.client.settings import settings as qt_settings

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def __map_response(idx:int,payload:json,response:json, status_code:int, headers:dict)->BatchOperationResult:

    result = BatchOperationResult(
            Index           = idx,
            DataPoints      = int(len(payload)),
            Success         = response.get('success'),
            FinishedAt      = datetime.now(),
            StatusCode      = status_code,
            ResponseHeaders = headers,
            ResponseErrors  = response.get('errors'),
            Duration        = response.get('duration'),
            BytesProcessed  = response.get('bytesProcessed')
        )

    return result.finalize()

async def __post_data_async(session:aiohttp.ClientSession, payload:json):
    
    async with session.post(
        url     = qt_settings()['services']['timeseries']['ingestion'], 
        json    = payload) as resp:
        
        response = await resp.json()
        return payload, response, resp.status, resp.headers

def __post_data_sync(idx:int, payload, http_client:sessions.BaseUrlSession, base_url, headers)->BatchOperationResult:
    
    if http_client.base_url is None:
        http_client.base_url = base_url

    response = http_client.post(
        url     = qt_settings()['services']['timeseries']['ingestion'], 
        headers = headers, 
        json    = payload, 
        verify  = False
    )

    return __map_response(
        idx         = idx, 
        payload     = payload, 
        response    = response.json(), 
        status_code = response.status_code, 
        headers     = response.headers).finalize()

async def ingest_async(timeserie:TimeSerie, http_client:sessions.BaseUrlSession, batch_size:int=500, ingest_properties:bool=True, operation_name:str = "Timeserie async ingestion"):
    """
        Send time serie to qant.io ingestion service
    
    """
    print("ingest async")
    
    batches = timeserie.to_batches(batch_size) #[payload[x:x+batch_size] for x in range(0, len(payload), batch_size)]
    
    headers = http_client.headers

    # Make OperationResult
    operation_result = OperationResult(OperationName = operation_name)
    
    base_url = http_client.base_url

    async with ClientSession(
        headers     = headers, 
        base_url    = base_url) as session:

        tasks = []
        
        for b in batches:
            tasks.append(asyncio.ensure_future(__post_data_async(session, b)))

        responses = await asyncio.gather(*tasks)
        
        for idx,response in enumerate(responses):
            operation_result.BatchOperations.append(__map_response(idx, response[0], response[1], response[2], dict(response[3])))

    return operation_result.finalize()
    

def ingest(timeserie:TimeSerie, http_client:sessions.BaseUrlSession, batch_size:int=500, n_jobs=1, operation_name:str = "Timeserie sync ingestion")->OperationResult:
    
    """
        Send time serie to qant.io ingestion service
    """
    
    # Set batchs
    batchs = timeserie.to_batches(batch_size, http_client.headers.get('ClientId'))
    pprint(batchs[0])
    
    # Set OperationResult
    operation_result = OperationResult(OperationName = operation_name)
    exit()
    
    # Run multiple jobs in //
    if n_jobs > 1:
        jobs = Parallel(n_jobs=n_jobs, backend='loky')(
            delayed(__post_data_sync)
                (idx, content, http_client, http_client.base_url, http_client.headers) for idx,content in enumerate(batchs)
            )

        operation_result.BatchOperations = jobs
        return operation_result.finalize()
    
    # Or run sequential jobs
    for idx, content in enumerate(batchs):
        operation_result.BatchOperations.append(__post_data_sync(
                idx         = idx, 
                payload     = content,
                http_client = http_client, 
                base_url    = http_client.base_url, 
                headers     = http_client.headers
            )
        )
       
    return operation_result.finalize()